# Generated by roxygen2: do not edit by hand

export("%not_in%")
export(impute)
export(is_it_normal)
export(nonlinear_bins)
export(normalize)
export(pretty_number)
export(pretty_percent)
export(probability_of_survival)
export(rm_bin_summary)
export(rmm)
export(season)
export(seqic_indicator_1)
export(seqic_indicator_10)
export(seqic_indicator_11)
export(seqic_indicator_12)
export(seqic_indicator_13)
export(seqic_indicator_2)
export(seqic_indicator_3)
export(seqic_indicator_4)
export(seqic_indicator_5)
export(seqic_indicator_6)
export(seqic_indicator_7)
export(seqic_indicator_8)
export(seqic_indicator_9)
export(small_count_label)
export(stat_sig)
export(theme_cleaner)
export(trauma_case_mix)
export(trauma_performance)
export(weekend)
import(rlang)
importFrom(glue,glue)
importFrom(infer,generate)
importFrom(lifecycle,badge)
importFrom(lifecycle,deprecated)
importFrom(nemsqar,nemsqa_binomial_confint)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(utils,head)
importFrom(utils,tail)
