#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom glue glue
#' @importFrom infer generate
#' @importFrom lifecycle badge
#' @importFrom lifecycle deprecated
#' @importFrom stats ks.test
#' @importFrom stats median
#' @importFrom stats na.omit
#' @importFrom stats quantile
#' @importFrom stats sd
#' @importFrom stats shapiro.test
#' @importFrom utils head
#' @importFrom utils tail
#' @importFrom nemsqar nemsqa_binomial_confint
## usethis namespace: end
NULL
