utils::globalVariables(c(
  "MTOS_distribution",
  "valid_trauma_types",
  "type",
  "column_name",
  "b0",
  "b1",
  "b2",
  "b3",
  "age_points",
  ":=",
  "bin_number",
  "bin_start",
  "bin_end",
  "TD_b",
  "TA_b",
  "midpoint",
  "AntiM_b",
  "N_b",
  "R_b",
  "EM_b",
  "numerator",
  "denominator",
  "RMM",
  "Ps_range",
  "survivors",
  "current_fraction",
  "smallest_val",
  "prob_death",
  "scale_factor",
  "bins",
  "bins_temp",
  "alive",
  "dead",
  "count",
  "sd",
  "bootstrap_RMM",
  "z_score",
  "AntiS_b",
  "ISS_Calculated",
  "Initial_Assessment_Calculated_RTS",
  "Patient_Age_Years",
  "bootstrap_CI",
  "bootstrap_RMM_LL",
  "bootstrap_RMM_UL",
  "population_CI",
  "population_RMM",
  "population_RMM_LL",
  "population_RMM_UL",
  "sd_bootstrap_RMM",
  "se_bootstrap_RMM",
  "stat_sig",
  "group_vars",
  "grouped_data",
  "grouped_stats",
  "Pred_Survivors_b",
  "Pred_Deaths_b",
  ".data",
  "Delayed_DC_2hr",
  "Delayed_DC_3hr",
  "Delayed_Decision_1hr",
  "Delayed_Decision_2hr",
  "Delayed_Decision_Discharge_1hr",
  "Delayed_Decision_Discharge_2hr",
  "arrive_greater_than_180",
  "full_minor",
  "full_major",
  "limited_minor",
  "limited_major",
  "denominator_10a",
  "denominator_10b",
  "denominator_10c",
  "denominator_11",
  "denominator_12",
  "denominator_13",
  "denominator_1a",
  "denominator_1b",
  "denominator_1c",
  "denominator_1d",
  "denominator_1e",
  "denominator_1f",
  "denominator_2",
  "denominator_3",
  "denominator_4a",
  "denominator_4b",
  "denominator_5a",
  "denominator_5b",
  "denominator_5c",
  "denominator_5d",
  "denominator_6",
  "denominator_7",
  "denominator_8_all",
  "denominator_8_risk",
  "denominator_9a_activations",
  "denominator_9a_activations_risk",
  "denominator_9a_all",
  "denominator_9a_risk",
  "denominator_9b_activations",
  "denominator_9b_activations_risk",
  "denominator_9b_all",
  "denominator_9b_risk",
  "denominator_9c_activations",
  "denominator_9c_activations_risk",
  "denominator_9c_all",
  "denominator_9c_risk",
  "denominator_9d_activations",
  "denominator_9d_activations_risk",
  "denominator_9d_all",
  "denominator_9d_risk",
  "denominator_9e_activations",
  "denominator_9e_activations_risk",
  "denominator_9e_all",
  "denominator_9e_risk",
  "denominator_9f_activations",
  "denominator_9f_activations_risk",
  "denominator_9f_all",
  "denominator_9f_risk",
  "full_activation",
  "ks.test",
  "limited_no_activation",
  "los_less_24_hrs",
  "lower_ci",
  "lower_ci_1d",
  "lower_ci_4a",
  "lower_ci_4b",
  "lower_ci_5a",
  "lower_ci_5b",
  "lower_ci_5c",
  "lower_ci_5d",
  "lower_ci_9a_activations",
  "lower_ci_9a_activations_risk",
  "lower_ci_9a_all",
  "lower_ci_9a_risk",
  "lower_ci_9b_activations",
  "lower_ci_9b_activations_risk",
  "lower_ci_9b_all",
  "lower_ci_9b_risk",
  "lower_ci_9c_activations",
  "lower_ci_9c_activations_risk",
  "lower_ci_9c_all",
  "lower_ci_9c_risk",
  "lower_ci_9d_activations",
  "lower_ci_9d_activations_risk",
  "lower_ci_9d_all",
  "lower_ci_9d_risk",
  "lower_ci_9e_activations",
  "lower_ci_9e_activations_risk",
  "lower_ci_9e_all",
  "lower_ci_9e_risk",
  "lower_ci_9f_activations",
  "lower_ci_9f_activations_risk",
  "lower_ci_9f_all",
  "lower_ci_9f_risk",
  "major_trauma",
  "median",
  "minor_trauma",
  "numerator_10a",
  "numerator_10b",
  "numerator_10c",
  "numerator_11",
  "numerator_12",
  "numerator_13",
  "numerator_1a",
  "numerator_1b",
  "numerator_1c",
  "numerator_1d",
  "numerator_1e",
  "numerator_1f",
  "numerator_2",
  "numerator_3",
  "numerator_4a",
  "numerator_4b",
  "numerator_5a",
  "numerator_5b",
  "numerator_5c",
  "numerator_5d",
  "numerator_6",
  "numerator_7",
  "numerator_8_all",
  "numerator_8_risk",
  "numerator_9a_activations",
  "numerator_9a_activations_risk",
  "numerator_9a_all",
  "numerator_9a_risk",
  "numerator_9b_activations",
  "numerator_9b_activations_risk",
  "numerator_9b_all",
  "numerator_9b_risk",
  "numerator_9c_activations",
  "numerator_9c_activations_risk",
  "numerator_9c_all",
  "numerator_9c_risk",
  "numerator_9d_activations",
  "numerator_9d_activations_risk",
  "numerator_9d_all",
  "numerator_9d_risk",
  "numerator_9e_activations",
  "numerator_9e_activations_risk",
  "numerator_9e_all",
  "numerator_9e_risk",
  "numerator_9f_activations",
  "numerator_9f_activations_risk",
  "numerator_9f_all",
  "numerator_9f_risk",
  "overtriage",
  "p.value",
  "quantile",
  "seqic_1d",
  "seqic_5a",
  "seqic_5b",
  "seqic_5c",
  "seqic_5d",
  "seqic_9a_activations",
  "seqic_9a_activations_risk",
  "seqic_9a_all",
  "seqic_9a_risk",
  "seqic_9b_activations",
  "seqic_9b_activations_risk",
  "seqic_9b_all",
  "seqic_9b_risk",
  "seqic_9c_activations",
  "seqic_9c_activations_risk",
  "seqic_9c_all",
  "seqic_9c_risk",
  "seqic_9d_activations",
  "seqic_9d_activations_risk",
  "seqic_9d_all",
  "seqic_9d_risk",
  "seqic_9e_activations",
  "seqic_9e_activations_risk",
  "seqic_9e_all",
  "seqic_9e_risk",
  "seqic_9f_activations",
  "seqic_9f_activations_risk",
  "seqic_9f_all",
  "seqic_9f_risk",
  "shapiro.test",
  "undertriage",
  "upper_ci",
  "upper_ci_1d",
  "upper_ci_4a",
  "upper_ci_4b",
  "upper_ci_5a",
  "upper_ci_5b",
  "upper_ci_5c",
  "upper_ci_5d",
  "upper_ci_9a_activations",
  "upper_ci_9a_activations_risk",
  "upper_ci_9a_all",
  "upper_ci_9a_risk",
  "upper_ci_9b_activations",
  "upper_ci_9b_activations_risk",
  "upper_ci_9b_all",
  "upper_ci_9b_risk",
  "upper_ci_9c_activations",
  "upper_ci_9c_activations_risk",
  "upper_ci_9c_all",
  "upper_ci_9c_risk",
  "upper_ci_9d_activations",
  "upper_ci_9d_activations_risk",
  "upper_ci_9d_all",
  "upper_ci_9d_risk",
  "upper_ci_9e_activations",
  "upper_ci_9e_activations_risk",
  "upper_ci_9e_all",
  "upper_ci_9e_risk",
  "upper_ci_9f_activations",
  "upper_ci_9f_activations_risk",
  "upper_ci_9f_all",
  "upper_ci_9f_risk",
  "validity_threshold",
  "validity",
  "ed_LOS"
))
